@@ -4,11 +4,13 @@ setAs("dspMatrix", "dsyMatrix",
       function(from) .Call(dspMatrix_as_dsyMatrix, from))
 
 ## dge <--> dsp   via  dsy
-setAs("dspMatrix", "dgeMatrix",
-      function(from) as(as(from, "dsyMatrix"), "dgeMatrix"))
 setAs("dgeMatrix", "dspMatrix",
       function(from) as(as(from, "dsyMatrix"), "dspMatrix"))
 
+## now have faster  "ddense" -> "dge":
+## setAs("dspMatrix", "dgeMatrix",
+##       function(from) as(as(from, "dsyMatrix"), "dgeMatrix"))
+
 ## S3-matrix <--> dsp   via  dsy
 setAs("dspMatrix", "matrix",
       function(from) as(as(from, "dsyMatrix"), "matrix"))
@@ -27,51 +29,39 @@ setMethod("rcond", signature(x = "dspMatrix", type = "missing"),
           .Call(dspMatrix_rcond, x, "O"),
           valueClass = "numeric")
 
-setMethod("%*%", signature(x = "dspMatrix", y = "dgeMatrix"),
-          function(x, y)
-          .Call(dspMatrix_matrix_mm, x, y, TRUE),
+setMethod("%*%", signature(x = "dspMatrix", y = "ddenseMatrix"),
+          function(x, y) .Call(dspMatrix_matrix_mm, x, y),
           valueClass = "dgeMatrix")
 
 setMethod("%*%", signature(x = "dspMatrix", y = "matrix"),
-          function(x, y)
-          .Call(dspMatrix_matrix_mm, x, y, FALSE),
+          function(x, y) .Call(dspMatrix_matrix_mm, x, y),
           valueClass = "dgeMatrix")
 
-##setMethod("%*%", signature(x = "dspMatrix", y = "numeric"),
-##          function(x, y)
-##          .Call(dspMatrix_matrix_mm, x, as.matrix(y), FALSE),
-##          valueClass = "dgeMatrix")
-
-setMethod("%*%", signature(x = "dspMatrix", y = "integer"),
-          function(x, y) {
-              storage.mode(y) <- "double"
-              .Call(dspMatrix_matrix_mm, x, as.matrix(y), FALSE)
-          }, valueClass = "dgeMatrix")
+## Should define multiplication from the right
 
 setMethod("solve", signature(a = "dspMatrix", b = "missing"),
 	  function(a, b, ...) .Call(dspMatrix_solve, a),
 	  valueClass = "dspMatrix")
 
 setMethod("solve", signature(a = "dspMatrix", b = "matrix"),
-	  function(a, b, ...)
-	  .Call(dspMatrix_matrix_solve, a, b, FALSE),
+	  function(a, b, ...) .Call(dspMatrix_matrix_solve, a, b),
 	  valueClass = "dgeMatrix")
 
-setMethod("solve", signature(a = "dspMatrix", b = "dgeMatrix"),
-	  function(a, b, ...)
-	  .Call(dspMatrix_matrix_solve, a, as(b,"matrix"), TRUE),
+setMethod("solve", signature(a = "dspMatrix", b = "ddenseMatrix"),
+	  function(a, b, ...) .Call(dspMatrix_matrix_solve, a, b),
 	  valueClass = "dgeMatrix")
 
 ##setMethod("solve", signature(a = "dspMatrix", b = "numeric"),
 ##	  function(a, b, ...)
-##	  .Call(dspMatrix_matrix_solve, a, as.matrix(b), FALSE),
+##	  .Call(dspMatrix_matrix_solve, a, as.matrix(b)),
 ##	  valueClass = "dgeMatrix")
 
-setMethod("solve", signature(a = "dspMatrix", b = "integer"),
-	  function(a, b, ...) {
-	      storage.mode(b) <- "double"
-	      .Call(dspMatrix_matrix_solve, a, as.matrix(b), FALSE)
-	  }, valueClass = "dgeMatrix")
+## No longer needed
+## setMethod("solve", signature(a = "dspMatrix", b = "integer"),
+## 	  function(a, b, ...) {
+## 	      storage.mode(b) <- "double"
+## 	      .Call(dspMatrix_matrix_solve, a, as.matrix(b))
+## 	  }, valueClass = "dgeMatrix")
 
 setMethod("norm", signature(x = "dspMatrix", type = "character"),
           function(x, type, ...) .Call(dspMatrix_norm, x, type),
