@@ -29,24 +29,24 @@ setAs("lsparseMatrix", "matrix",
 ## selections.  Coerce a ddensematrix argument to a lsparseMatrix.
 
 setMethod("%*%", signature(x = "lsparseMatrix", y = "ldenseMatrix"),
-          function(x, y) callGeneric(x, as(y, "lsparseMatrix")))
+	  function(x, y) x %*% as(y, "lsparseMatrix"))
 
 setMethod("%*%", signature(x = "ldenseMatrix", y = "lsparseMatrix"),
-          function(x, y) callGeneric(as(x, "lsparseMatrix"), y))
+	  function(x, y) as(x, "lsparseMatrix") %*% y)
 
 setMethod("crossprod", signature(x = "lsparseMatrix", y = "ldenseMatrix"),
-          function(x, y = NULL) callGeneric(x, as(y, "lsparseMatrix")))
+	  function(x, y = NULL) crossprod(x, as(y, "lsparseMatrix")))
 
 setMethod("crossprod", signature(x = "ldenseMatrix", y = "lsparseMatrix"),
-          function(x, y = NULL) callGeneric(as(x, "lsparseMatrix"), y))
+	  function(x, y = NULL) crossprod(as(x, "lsparseMatrix"), y))
 
 ## and coerce lsparse* to lgC*
 setMethod("%*%", signature(x = "lsparseMatrix", y = "lsparseMatrix"),
-          function(x, y) callGeneric(as(x, "lgCMatrix"), as(y, "lgCMatrix")))
+	  function(x, y) as(x, "lgCMatrix") %*% as(y, "lgCMatrix"))
 
 setMethod("crossprod", signature(x = "lsparseMatrix", y = "lsparseMatrix"),
-          function(x, y = NULL)
-          callGeneric(as(x, "lgCMatrix"), as(y, "lgCMatrix")))
+	  function(x, y = NULL)
+	  crossprod(as(x, "lgCMatrix"), as(y, "lgCMatrix")))
 
 
 setMethod("all", signature(x = "lsparseMatrix"),
