@@ -5,9 +5,11 @@
 setAs("nsTMatrix", "matrix",
       function(from) as(as(from, "ngTMatrix"), "matrix"))
 
+setAs("nsTMatrix", "ngCMatrix", # for diag
+      function(from) as(as(from, "nsCMatrix"), "ngCMatrix"))
+
 setAs("nsTMatrix", "ngTMatrix",
-      function(from) new("ngTMatrix", i = from@i, j = from@j,
-                         Dim = from@Dim, Dimnames = from@Dimnames))
+      function(from) .Call(nsTMatrix_as_ngTMatrix, from))
 
 setAs("nsTMatrix", "dsTMatrix",
       function(from) new("dsTMatrix", i = from@i, j = from@j,
@@ -15,6 +17,10 @@ setAs("nsTMatrix", "dsTMatrix",
                          diag = from@diag,
                          Dim = from@Dim, Dimnames = from@Dimnames))
 
+setAs("nsTMatrix", "nsyMatrix",
+      function(from) .Call(nsTMatrix_as_nsyMatrix, from))
+
+
 ## untested:
 setMethod("image", "nsTMatrix",
           function(x, ...) {
