@@ -1,5 +1,6 @@
 			/* Sparse matrices in compressed column-oriented form */
 #include "Csparse.h"
+#include "Tsparse.h"
 #include "chm_common.h"
 
 SEXP Csparse_validate(SEXP x)
@@ -170,7 +171,7 @@ SEXP Csparse_symmetric_to_general(SEXP x)
 SEXP Csparse_general_to_symmetric(SEXP x, SEXP uplo)
 {
     CHM_SP chx = AS_CHM_SP(x), chgx;
-    int uploT = (*CHAR(asChar(uplo)) == 'U') ? 1 : -1;
+    int uploT = (*CHAR(STRING_ELT(uplo,0)) == 'U') ? 1 : -1;
     int Rkind = (chx->xtype != CHOLMOD_PATTERN) ? Real_kind(x) : 0;
     R_CheckStack();
 
@@ -202,40 +203,81 @@ SEXP Csparse_transpose(SEXP x, SEXP tri)
 
 SEXP Csparse_Csparse_prod(SEXP a, SEXP b)
 {
-    CHM_SP cha = AS_CHM_SP(a), chb = AS_CHM_SP(b);
-    CHM_SP chc = cholmod_ssmult(cha, chb, 0, cha->xtype, 1, &c);
+    CHM_SP
+	cha = AS_CHM_SP(Csparse_diagU2N(a)),
+	chb = AS_CHM_SP(Csparse_diagU2N(b)),
+	chc = cholmod_ssmult(cha, chb, /*out_stype:*/ 0,
+			     cha->xtype, /*out sorted:*/ 1, &c);
+    const char *cl_a = class_P(a), *cl_b = class_P(b);
+    char diag[] = {'\0', '\0'};
+    int uploT = 0;
     SEXP dn = allocVector(VECSXP, 2);
     R_CheckStack();
 
+    /* Preserve triangularity and even unit-triangularity if appropriate.
+     * Note that in that case, the multiplication itself should happen
+     * faster.  But there's no support for that in CHOLMOD */
+
+    /* UGLY hack -- rather should have (fast!) C-level version of
+     *       is(a, "triangularMatrix") etc */
+    if (cl_a[1] == 't' && cl_b[1] == 't')
+	/* FIXME: fails for "Cholesky","BunchKaufmann"..*/
+	if(*uplo_P(a) == *uplo_P(b)) { /* both upper, or both lower tri. */
+	    uploT = (*uplo_P(a) == 'U') ? 1 : -1;
+	    if(*diag_P(a) == 'U' && *diag_P(b) == 'U') { /* return UNIT-triag. */
+		/* "remove the diagonal entries": */
+		chm_diagN2U(chc, uploT, /* do_realloc */ FALSE);
+		diag[0]= 'U';
+	    }
+	    else diag[0]= 'N';
+	}
     SET_VECTOR_ELT(dn, 0,	/* establish dimnames */
 		   duplicate(VECTOR_ELT(GET_SLOT(a, Matrix_DimNamesSym), 0)));
     SET_VECTOR_ELT(dn, 1,
 		   duplicate(VECTOR_ELT(GET_SLOT(b, Matrix_DimNamesSym), 1)));
-    return chm_sparse_to_SEXP(chc, 1, 0, 0, "", dn);
+    return chm_sparse_to_SEXP(chc, 1, uploT, /*Rkind*/0, diag, dn);
 }
 
 SEXP Csparse_Csparse_crossprod(SEXP a, SEXP b, SEXP trans)
 {
     int tr = asLogical(trans);
-    CHM_SP cha = AS_CHM_SP(a), chb = AS_CHM_SP(b), chTr, chc;
+    CHM_SP
+	cha = AS_CHM_SP(Csparse_diagU2N(a)),
+	chb = AS_CHM_SP(Csparse_diagU2N(b)),
+	chTr, chc;
+    const char *cl_a = class_P(a), *cl_b = class_P(b);
+    char diag[] = {'\0', '\0'};
+    int uploT = 0;
     SEXP dn = allocVector(VECSXP, 2);
     R_CheckStack();
 
     chTr = cholmod_transpose((tr) ? chb : cha, chb->xtype, &c);
     chc = cholmod_ssmult((tr) ? cha : chTr, (tr) ? chTr : chb,
-			 0, cha->xtype, 1, &c);
+			 /*out_stype:*/ 0, cha->xtype, /*out sorted:*/ 1, &c);
     cholmod_free_sparse(&chTr, &c);
 
+    /* Preserve triangularity and unit-triangularity if appropriate;
+     * see Csparse_Csparse_prod() for comments */
+    if (cl_a[1] == 't' && cl_b[1] == 't')
+	if(*uplo_P(a) != *uplo_P(b)) { /* one 'U', the other 'L' */
+	    uploT = (*uplo_P(b) == 'U') ? 1 : -1;
+	    if(*diag_P(a) == 'U' && *diag_P(b) == 'U') { /* return UNIT-triag. */
+		chm_diagN2U(chc, uploT, /* do_realloc */ FALSE);
+		diag[0]= 'U';
+	    }
+	    else diag[0]= 'N';
+	}
+
     SET_VECTOR_ELT(dn, 0,	/* establish dimnames */
 		   duplicate(VECTOR_ELT(GET_SLOT(a, Matrix_DimNamesSym), (tr) ? 0 : 1)));
     SET_VECTOR_ELT(dn, 1,
 		   duplicate(VECTOR_ELT(GET_SLOT(b, Matrix_DimNamesSym), (tr) ? 0 : 1)));
-    return chm_sparse_to_SEXP(chc, 1, 0, 0, "", dn);
+    return chm_sparse_to_SEXP(chc, 1, uploT, /*Rkind*/0, diag, dn);
 }
 
 SEXP Csparse_dense_prod(SEXP a, SEXP b)
 {
-    CHM_SP cha = AS_CHM_SP(a);
+    CHM_SP cha = AS_CHM_SP(Csparse_diagU2N(a));
     SEXP b_M = PROTECT(mMatrix_as_dgeMatrix(b));
     CHM_DN chb = AS_CHM_DN(b_M);
     CHM_DN chc = cholmod_allocate_dense(cha->nrow, chb->ncol, cha->nrow,
@@ -255,7 +297,7 @@ SEXP Csparse_dense_prod(SEXP a, SEXP b)
 
 SEXP Csparse_dense_crossprod(SEXP a, SEXP b)
 {
-    CHM_SP cha = AS_CHM_SP(a);
+    CHM_SP cha = AS_CHM_SP(Csparse_diagU2N(a));
     SEXP b_M = PROTECT(mMatrix_as_dgeMatrix(b));
     CHM_DN chb = AS_CHM_DN(b_M);
     CHM_DN chc = cholmod_allocate_dense(cha->ncol, chb->ncol, cha->ncol,
@@ -273,20 +315,26 @@ SEXP Csparse_dense_crossprod(SEXP a, SEXP b)
     return chm_dense_to_SEXP(chc, 1, 0, dn);
 }
 
-/* Computes   x'x  or  x x'  -- see Csparse_Csparse_crossprod above for  x'y and x y' */
+/* Computes   x'x  or  x x' -- *also* for Tsparse (triplet = TRUE)
+   see Csparse_Csparse_crossprod above for  x'y and x y' */
 SEXP Csparse_crossprod(SEXP x, SEXP trans, SEXP triplet)
 {
     int trip = asLogical(triplet),
 	tr   = asLogical(trans); /* gets reversed because _aat is tcrossprod */
-    CHM_TR cht = trip ? AS_CHM_TR(x) : (CHM_TR) NULL;
+    CHM_TR cht = trip ? AS_CHM_TR(Tsparse_diagU2N(x)) : (CHM_TR) NULL;
     CHM_SP chcp, chxt,
-	chx = trip ? cholmod_triplet_to_sparse(cht, cht->nnz, &c) : AS_CHM_SP(x);
+	chx = (trip ?
+	       cholmod_triplet_to_sparse(cht, cht->nnz, &c) :
+	       AS_CHM_SP(Csparse_diagU2N(x)));
     SEXP dn = PROTECT(allocVector(VECSXP, 2));
     R_CheckStack();
 
     if (!tr) chxt = cholmod_transpose(chx, chx->xtype, &c);
     chcp = cholmod_aat((!tr) ? chxt : chx, (int *) NULL, 0, chx->xtype, &c);
-    if(!chcp) error(_("Csparse_crossprod(): error return from cholmod_aat()"));
+    if(!chcp) {
+	UNPROTECT(1);
+	error(_("Csparse_crossprod(): error return from cholmod_aat()"));
+    }
     cholmod_band_inplace(0, chcp->ncol, chcp->xtype, chcp, &c);
     chcp->stype = 1;
     if (trip) cholmod_free_sparse(&chx, &c);
@@ -348,10 +396,14 @@ SEXP Csparse_band(SEXP x, SEXP k1, SEXP k2)
 
 SEXP Csparse_diagU2N(SEXP x)
 {
-    if (*diag_P(x) != 'U') {/* "trivially fast" when there's no 'diag' slot at all */
+    const char *cl = class_P(x);
+    /* dtCMatrix, etc; [1] = the second character =?= 't' for triangular */
+    if (cl[1] != 't' || *diag_P(x) != 'U') {
+	/* "trivially fast" when not triangular (<==> no 'diag' slot),
+	   or not *unit* triangular */
 	return (x);
     }
-    else {
+    else { /* unit triangular (diag='U'): "fill the diagonal" & diag:= "N" */
 	CHM_SP chx = AS_CHM_SP(x);
 	CHM_SP eye = cholmod_speye(chx->nrow, chx->ncol, chx->xtype, &c);
 	double one[] = {1, 0};
@@ -366,6 +418,30 @@ SEXP Csparse_diagU2N(SEXP x)
     }
 }
 
+SEXP Csparse_diagN2U(SEXP x)
+{
+    const char *cl = class_P(x);
+    /* dtCMatrix, etc; [1] = the second character =?= 't' for triangular */
+    if (cl[1] != 't' || *diag_P(x) != 'N') {
+	/* "trivially fast" when not triangular (<==> no 'diag' slot),
+	   or already *unit* triangular */
+	return (x);
+    }
+    else { /* triangular with diag='N'): now drop the diagonal */
+	/* duplicate, since chx will be modified: */
+	CHM_SP chx = AS_CHM_SP(duplicate(x));
+	int uploT = (*uplo_P(x) == 'U') ? 1 : -1,
+	    Rkind = (chx->xtype != CHOLMOD_PATTERN) ? Real_kind(x) : 0;
+	R_CheckStack();
+
+	chm_diagN2U(chx, uploT, /* do_realloc */ FALSE);
+
+	return chm_sparse_to_SEXP(chx, /*dofree*/ 0/* or 1 ?? */,
+				  uploT, Rkind, "U",
+				  GET_SLOT(x, Matrix_DimNamesSym));
+    }
+}
+
 SEXP Csparse_submatrix(SEXP x, SEXP i, SEXP j)
 {
     CHM_SP chx = AS_CHM_SP(x);
@@ -393,8 +469,8 @@ SEXP Csparse_MatrixMarket(SEXP x, SEXP fname)
     if (!f)
 	error(_("failure to open file \"%s\" for writing"),
 	      CHAR(asChar(fname)));
-    if (!cholmod_write_sparse(f, AS_CHM_SP(x), (CHM_SP)NULL,
-			      (char*) NULL, &c))
+    if (!cholmod_write_sparse(f, AS_CHM_SP(Csparse_diagU2N(x)),
+			      (CHM_SP)NULL, (char*) NULL, &c))
 	error(_("cholmod_write_sparse returned error code"));
     fclose(f);
     return R_NilValue;
